## SISCER 2021: Mixed-effects Models for Longitudinal Data Analysis
## Benjamin French; b.french@vumc.org

install.packages(c('lme4', 'dplyr', 'ggplot2', 'JM'), 
                 repos='https://cloud.r-project.org')

library(lme4)
library(dplyr)
library(ggplot2)
library(JM)

theme_set(theme_bw())
mytheme <- function(...){
  theme(panel.grid.minor=element_blank(),
        plot.margin=unit(c(0.5, 0.5, 0.5, 0.5), 'lines'),
        axis.title.x=element_text(size=12, vjust=0),
        axis.title.y=element_text(size=12, vjust=1, angle=90),
        legend.title=element_text(size=12, face='bold'),
        legend.text=element_text(size=12),
        legend.position='top', legend.direction='horizontal', 
        legend.key=element_blank(),
        legend.key.width=unit(1.5,"cm"),
        legend.key.height=unit(0.35,'cm'),
        strip.text.y=element_text(size=12))}

?pbc2
data(pbc2)
head(pbc2, 11)
str(pbc2)

pbc2 %>%
  group_by(id) %>%
  summarize(m=sum(!is.na(albumin))) %>%
  summarize(min(m), median(m), max(m))

subset(pbc2, !is.na(albumin)) %>% 
  count(id, name='m') -> num
with(num, table(m))

ggplot(subset(pbc2, is.element(id, 1:16)), aes(year, albumin)) +
  facet_wrap( ~ id) +
  geom_vline(aes(xintercept=years, color=factor(status!='dead')), lwd=1) + 
  geom_point() +
  geom_smooth(se=FALSE, method='lm') + 
  scale_x_continuous(name='Time, years', limits=c(0,15)) +
  scale_y_continuous(name='Serum albumin, g/dl') + 
  scale_colour_manual(name='Status', breaks=c(FALSE, TRUE), values=c('red', 'darkgray'),
                      labels=c('Death', 'Censored')) +
  mytheme()

?geom_smooth

pbc2 <- within(pbc2, {
  levels(drug) <- c('Placebo', 'Treatment')})

ggplot(subset(pbc2, year==0), aes(x=albumin)) +
  geom_histogram(color='black', fill='white') +
  facet_wrap(~ drug) +
  scale_x_continuous(name='Serum albumin, g/dl') +
  scale_y_continuous(name='Count') +
  mytheme()

ggplot(subset(pbc2, albumin<6), aes(year, albumin, group=id)) +
  geom_line(aes(color=drug), alpha=0.4) +
  geom_smooth(aes(year, albumin, group=drug, color=drug), se=FALSE, size=1.5) +
  scale_x_continuous(name='Time, years', limits=c(0,15)) +
  scale_y_continuous(name='Serum albumin, g/dl') + 
  scale_color_manual(name='Intervention', values=c('#00BFC4', '#F8766D')) +
  mytheme()

pbc2 %>% 
  group_by(id) %>% 
  mutate(obs_num = row_number()) %>%
  ungroup() %>%
  as.data.frame() -> pbc2

pbc2_wide <- reshape(subset(pbc2, select=c(id, obs_num, albumin)),
                     v.names='albumin', idvar='id', timevar='obs_num', direction='wide')
head(pbc2_wide, 2)

ntime <- 8
rmat <- with(pbc2_wide, cbind(albumin.1, albumin.2, albumin.3, albumin.4, albumin.5, albumin.6, albumin.7, albumin.8))

vmat <- nmat <- covmatj <- covmatk <- matrix(0, nrow=ntime, ncol=ntime)
rownames(vmat) <- colnames(vmat) <- 1:ntime
rownames(nmat) <- colnames(nmat) <- 1:ntime

for(j in 1:ntime){
  for(k in 1:ntime){
    nmat[j,k] <- sum(!is.na(rmat[,j]*rmat[,k]))
    vmat[j,k] <- cov(rmat[,j],rmat[,k],use="pairwise.complete.obs")
    if(nmat[j,k]>0) covmatj[j,k] <- cov(rmat[,j][!is.na(rmat[,j]*rmat[,k])],rmat[,j][!is.na(rmat[,j]*rmat[,k])],use="pairwise.complete.obs")
    if(nmat[j,k]>0) covmatk[j,k] <- cov(rmat[,k][!is.na(rmat[,j]*rmat[,k])],rmat[,k][!is.na(rmat[,j]*rmat[,k])],use="pairwise.complete.obs")
  }}

cmat <- vmat/sqrt(covmatj)/sqrt(covmatk)

round(diag(vmat),2)
round(cmat, 2)
nmat

?lmer
?glmer

# Main effects of treatment and time, random intercepts
lme_m1 <- lmer(formula=albumin ~ (1 | id) + drug + year, data=pbc2)
summary(lme_m1)

# Interaction between treatment and time, random intercepts
lme_m2 <- lmer(formula=albumin ~ (1 | id) + drug*year, data=pbc2)
summary(lme_m2)

# Treatment-specific time effects, random intercepts
lme_m3 <- lmer(formula=albumin ~ (1 | id) + drug:year, data=pbc2)
summary(lme_m3)

# Treatment-specific time effects, random intercepts + slopes
lme_m4 <- lmer(formula=albumin ~ (year | id) + drug:year, data=pbc2)
summary(lme_m4)

# 95% confidence intervals
confint(lme_m4, parm='beta_')

# Likelihood ratio test for inclusion of random slopes
anova(lme_m4, lme_m3)

methods(class="merMod")

?residuals.merMod
?fitted.merMod

plotResid <- function(x, y, col.lowess='red', ...){
  plot(x, y, ...)
  abline(h=0, col='gray', lwd=2)
  lines(lowess(x, y), col=col.lowess, lwd=2)}
plotQQ <- function(x, ...){
  qqnorm(x, ...)
  qqline(x, col='gray', lwd=2)}

par(mfrow=c(2,2), mar=c(5,5,2,1))
plotResid(fitted(lme_m4), residuals(lme_m4), xlab='Fitted values', ylab='Residuals')
plotQQ(residuals(lme_m4), main='Residuals')
plotQQ(ranef(lme_m4)$id[,1], main='Random intercepts')
plotQQ(ranef(lme_m4)$id[,2], main='Random slopes')

library(geepack)
gee_m5 <- geeglm(formula=albumin ~ drug:year + drug:I(year^2), family='gaussian',
                 data=pbc2, id=id, corstr='exchangeable', scale.fix=TRUE)
anova(gee_m5)

lme_m5 <- lmer(formula=albumin ~ (year | id) + drug:year + drug:I(year^2), data=pbc2)
anova(lme_m5, lme_m4)

print(sessionInfo(), locale = FALSE)
